/***************************************************************************
 *   Copyright (C) 2008 by C J Plooy                                       *
 *   cornwarecjp@lavabit.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ORBITMFD_H
#define ORBITMFD_H

#include "orbitersdk.h"
#include "kost.h"


//Some settings:
//Choose one of these:
#define ORBIT_MFD_G_INDICATOR_COMPATIBLE
#undef  ORBIT_MFD_G_INDICATOR_CJP



class COrbitMFD : public MFD
{
public:

	//Interface with Orbiter:
	COrbitMFD(DWORD w, DWORD h, VESSEL *vessel);
	~COrbitMFD();
	static int MsgProc(UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam);

	void Update (HDC hDC);

	char *ButtonLabel(int bt);
	int ButtonMenu(const MFDBUTTONMENU **menu) const;

	bool ConsumeButton(int bt, int event);
	bool ConsumeKeyBuffered (DWORD key);

	void WriteStatus (FILEHANDLE scn) const;
	void ReadStatus (FILEHANDLE scn);

	void StoreStatus() const;
	void RecallStatus();

	//Button response functions:
	void btnREF();
	void btnAR();
	void btnTGT();
	void btnNT();
	void btnMOD();
	void btnFRM();
	void btnPRJ();
	void btnDST();
	void btnHUD();

	void autoReference();

	//State of the MFD:
	OBJHANDLE m_Reference;
	OBJHANDLE m_Target;
	bool m_ShowAlt;
	enum eFrame      {eFEcliptic, eFEquator,     eFMax} m_Frame;
	enum eProjection {ePFrame, ePShip, ePTarget, ePMax} m_Projection;

	//Non-stored state:
	enum eMode {eMBoth, eMList, eMGraphics, eMMax} m_Mode;

protected:
	VESSEL *m_Vessel;
	unsigned int m_Width, m_Height;


	struct SDisplayData
	{
		char *title;
		kostStateVector state;
		kostElements    elements;
		kostOrbitParam  params;

		kostOrbitShape  shape;
	};

	//Temporary results:
	SDisplayData m_VesselData, m_TargetData;
	kostReal m_GraphicsScale, m_GraphicsRange;

	void calcDisplayData();
	double calcGContribution(bool &makeRed) const;

	void calcScaleAndRange();
	double m_Scale;

	//From Frm coordinates to Prj coordinates
	//This does not transform orbital elements,
	//only state.pos and shape.
	void projectData(SDisplayData &data);

	void drawDisplayData(HDC hDC, int x, int y, const SDisplayData &data);

	void drawOrbitShape(HDC hDC, COLORREF color, const SDisplayData &data);
};


#endif

